"""Input key constants to decouple game logic from backend (Pygame)."""

# Standard ASCII
SPACE = 32
RETURN = 13
ESCAPE = 27

# Arrows
UP = 1073741906
DOWN = 1073741905
RIGHT = 1073741903
LEFT = 1073741904

# Letters
W = 119
A = 97
S = 115
D = 100
F = 102
